'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = plugin;

var _assert = require('assert');

var _assert2 = _interopRequireDefault(_assert);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _mkdirp = require('mkdirp');

var _mkdirp2 = _interopRequireDefault(_mkdirp);

var _objectAssign = require('object-assign');

var _objectAssign2 = _interopRequireDefault(_objectAssign);

var _foreach = require('foreach');

var _foreach2 = _interopRequireDefault(_foreach);

var _transformObjectExpressionIntoStyleSheetObject = require('./transformObjectExpressionIntoStyleSheetObject');

var _transformObjectExpressionIntoStyleSheetObject2 = _interopRequireDefault(_transformObjectExpressionIntoStyleSheetObject);

var _transformStyleSheetObjectIntoSpecification = require('./transformStyleSheetObjectIntoSpecification');

var _transformStyleSheetObjectIntoSpecification2 = _interopRequireDefault(_transformStyleSheetObjectIntoSpecification);

var _generateClassName = require('./generateClassName');

var _generateClassName2 = _interopRequireDefault(_generateClassName);

var _buildCSS = require('./buildCSS');

var _buildCSS2 = _interopRequireDefault(_buildCSS);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var KEY = '__cssinjs';

var DEFAULT_OPTIONS = {
  vendorPrefixes: false,
  minify: false,
  compressClassNames: false,
  mediaMap: {},
  context: null,
  cacheDir: 'tmp/cache/',
  bundleFile: 'bundle.css'
};

function plugin(context) {
  context[KEY] = {};

  return {
    visitor: visitor(context)
  };
}

function visitor(context) {
  var t = context.types;

  return {
    Program: {
      enter: function enter() {
        var filename = _path2.default.relative(process.cwd(), this.file.opts.filename);
        var options = buildOptions(this.opts, filename);

        this.cssInJS = { filename: filename, options: options, stylesheets: {} };
      },
      exit: function exit() {
        /* istanbul ignore if */
        if (this.done) return;

        var css = (0, _buildCSS2.default)(this.cssInJS.stylesheets, this.cssInJS.options);

        context[KEY][this.cssInJS.filename] = this.file.metadata.css = css;

        var bundleCSS = '';

        (0, _foreach2.default)(context[KEY], function (css, filename) {
          if (css.length) {
            bundleCSS += css;
          }
        });

        if (bundleCSS.length && this.cssInJS.options.bundleFile) {
          var bundleFile = _path2.default.join(process.cwd(), this.cssInJS.options.bundleFile);

          _mkdirp2.default.sync(_path2.default.dirname(bundleFile));
          _fs2.default.writeFileSync(bundleFile, bundleCSS, { encoding: 'utf8' });
        }

        this.done = true;
      }
    },

    CallExpression: function CallExpression(path) {
      if (!t.isIdentifier(path.node.callee, { name: 'cssInJS' })) {
        return;
      }

      (0, _assert2.default)(t.isVariableDeclarator(path.parentPath.node), 'return value of cssInJS(...) must be assigned to a variable');

      var sheetId = path.parentPath.node.id.name;
      var expr = path.node.arguments[0];

      (0, _assert2.default)(expr, 'cssInJS(...) call is missing an argument');

      var obj = (0, _transformObjectExpressionIntoStyleSheetObject2.default)(expr, this.cssInJS.options.context);
      var sheet = (0, _transformStyleSheetObjectIntoSpecification2.default)(obj);

      this.cssInJS.stylesheets[sheetId] = sheet;

      var gcnOptions = (0, _objectAssign2.default)({}, this.cssInJS.options, { prefixes: [this.cssInJS.filename, sheetId] });

      var properties = Object.keys(sheet).reduce(function (memo, styleId) {
        return memo.concat(t.objectProperty(t.identifier(styleId), t.stringLiteral((0, _generateClassName2.default)(styleId, gcnOptions))));
      }, []);

      path.replaceWith(t.objectExpression(properties));
    }
  };
}

var contextFileCache = {};

function buildOptions(options, filename) {
  options = (0, _objectAssign2.default)({}, DEFAULT_OPTIONS, options, { filename: filename });

  if (typeof options.context === 'string') {
    var file = _path2.default.resolve(options.context);

    if (typeof contextFileCache[file] === 'undefined') {
      contextFileCache[file] = require(file);
    }

    options.context = contextFileCache[file];
  }

  return options;
}
module.exports = exports['default'];
